#################################################################################
# Created       : Martin Foster
# Modified      : 18/09/2002
#################################################################################
#
# Filter - A filtering system that allows for safer handling of posts  
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Filter;

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	my $class = shift;
	my $self  = { };

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


#####################
# SafeHTML 
#
# While the name may be slightly misleading as the HTML can still be malicious to a degree
# this subroutine will aim to make it safer.   Note however, that the only safe HTML is no
# HTML.

sub SafeHTML 	 						# String to handle 
{
	#####################
	# Data members
	my $self    = shift;					# Reference to self
	my $str     = shift;					# String name

	my $count;						# Counter
	my $tag;						# Singular tag
	
	my %tags;						# Tags encoutered


	#####################
	# Program area
        # Title filters

	# Comments
	$str =~ s/<!-|-->|<COMMENT>|<\/COMMENT>//gsi;

	# Standard filtering 
	$str =~ s/<APPLET.*?>|<\/APPLET>|<BODY.*?>|<\/BODY>|<BGSOUND>|<EMBED.*?>|<\/EMBED>|<FORM.*?>|<\/FORM>|<IMG.*?>|<IMAGE.*?>|<INPUT.*?>//gsi;
	$str =~ s/<HTML.*?>|<\/HTML>|<OPTION.*?>|<SCRIPT.*?>|<TABLE.*?>|<\/TABLE>|<TEXTAREA.*?>|<\/TEXTAREA>|<TITLE.*?>|<\/TITLE>//gsi;

	# Close tags
	while ($str =~ m/<(\w.*?)(\s|>)/gs)
	{
		# Skips
		next if ($1 =~ /^BR/i);
		next if ($1 =~ /^HR/i);
		next if ($1 =~ /^IMG/i);
		next if ($1 =~ /^IMAGE/i);

		# Track tags
		if (defined($tags{$1})) { $tags{$1}++; }	# Increment
		else { $tags{$1} = 1; }				# Initialize
	}

	# Determine need for additions
	foreach $tag (keys(%tags))
	{
		# Reset counter
		$count = 0;

		# Count closing tags
		while ($str =~ m/<\/$tag>/gs)
		{ $count++; }

		# Add whatever is needed
		for ($count; $count < $tags{$tag}; $count++)
		{ $str = "$str</$tag>"; }	
	}

	# Return
	return $str;
}


#####################
# Strip HTML 
#
# Strip HTML is a harsher addtion to the system.  As this is designed to scrub
# much more of the HTML then the above version.  This is of course to prevent 
# people from adding HTML in things that are useful like a puppet name.

sub StripHTML 	 						# String to handle 
{
	#####################
	# Data members
	my $self    = shift;					# Reference to self
	my $str     = shift;					# String name


	#####################
	# Program area
        # Title filters

	# Html and special
	$str =~ s/<[^>]*>//gs;
	$str =~ s/\&.*?\;//gs;

	# Comments
	$str =~ s/<!-|-->//gs;

	# Return
	return $str;
}


#####################
# Sanity
#
# Will make sure that no single set of characters is longer then a certain amount
# as to prevent the horizontal scroll bar from kicking in.   As a result, hyperlinks
# must be dealt with specially in order to prevent their truncation.

sub MakeSane
{
	#####################
	# Data members
	my $self = shift;					# Self referencing scalar
	my $str  = shift;					# String to look into
	my $num  = shift;					# Number to truncate with

	my $tmp;

	my $cnt = 0;						# Counter
	my $eml;						# Amount of Email
	my $lnk;						# Amount of links

	my $fin;						# Final string
	my $org;						# Original position
	my $pos;						# Position
	my $lnt;						# Determined length

	my @eml;						# Email addresss
	my @lnk; 						# Link holder


	#####################
	# Program area

	# Hyperlink handling
	# Loop and find
	# Hyperlinks
	while ($str =~ /(http:\/\/.*?)(\s|$)/)
	{
		# Assign
		$org = $1;
	 	$lnk[$cnt] = $1;			

		# Escape then replace
		$org =~ s/(\W)/\\$1/gs;
		$str =~ s/$org/LINK$cnt/gsi;

		# Incremenet
		$cnt++;
	}

	# Assign
	$lnk = $cnt;
	$cnt = 0;


	# Email addresses
	while ($str =~ m/([\w\.\-]*?\@[\w\.\-]*?\..*?)(\s|$)/)
	{
		# Assign
		$org = $1;
	 	$eml[$cnt] = $1;			

		# Escape then replace
		$org =~ s/(\W)/\\$1/gs;
		$str =~ s/$org/MAIL$cnt/gsi;

		# Incremenet
		$cnt++;
	}

	# Assign
	$eml = $cnt;
	

	# All long strings
	# Loop and find
	while ($str =~ m/(\S+)/gs)
	{
		# Reset
		$pos = 0;

		# Determine need to correct
		if (length($1) > $num)
		{
			# Length of string
			$lnt = length($1);


			# Initial assignmants
			$org =  $1;
			$fin =  substr($1, 0, $num);
			$pos += $num;


			# Loop
			# Ensure truncation
			while ($pos < $lnt)
			{
				# Truncate and add space
				$fin = $fin . ' ' . substr($1, $pos, $num);

				# Increment
				$pos += $num;
			}
		
			# Escape for replacement
			$org =~ s/(\W)/\\$1/gs;

			# Replace values
			$str =~ s/$org/$fin/gs;
		}
	}


	# Re-establish sanity
	# Return hyperlinks
	for ($cnt=0; $cnt < $lnk; $cnt++)
	{
		# Replace with orginal value
		$str =~ s/LINK$cnt/$lnk[$cnt]/gs;
	}	

	# Email addresses
	for ($cnt=0; $cnt < $eml; $cnt++)
	{
		# Replace with orginal value
		$str =~ s/MAIL$cnt/$eml[$cnt]/gs;
	}

	# Return for use
	return $str;
}

1;								# Return true
